-- world lua

world = class:new()

local maplist = { "map0", "map1", "map2", "map3", "map4", "map5", "map6", "menu" } -- Here get the .map loaded in / menu "hack" returns to mainmain

function world:init()
	
	local g = love.graphics

	require "lua/objects/entity"
	require "lua/objects/tile"
	
	world:changeMap( "menubackground" )
	
-- lighting system
	self.lightFlickerTimer = 0
	self.lightUpdateTimer = 0
	lightmask = g.newCanvas( map.width, map.height )
	lightmaskimg = g.newImage( "gfx/lightmask.png" )
	
 -- to completly disable single entitys make sure you remove it out if this table and comment it out in the function below.
	self.entityList = { grass, jumppad, spikepad, torch, spikewheel, spike, water, sign, checkpoint, conveyor }

end

function world:loadEntitys()

	local g = love.graphics

-- load entitys
	require "lua/objects/spikewheel"
	spikewheel:init( 24, g.newImage( "gfx/game/spikewheel1.png" ), 100 )
	spikewheel:addParticles( "collide", tile.size / 2, tile.size / 2, 220, 50, 50, 150 )
	spikewheel:setHitbox( 0, 0, tile.size, tile.size )
	
	require "lua/objects/water"
	water:init( 26, g.newImage( "gfx/game/water.png" ), 100 ) 
	water:addParticles( "collide", 0, 0, 0, 146, 248, 255 )
	water:setHitbox( 8, 2, tile.size - 16, tile.size - 2 )
	
	require "lua/objects/torch"		
	torch:init( 23, g.newImage( "gfx/game/torch1.png" ), 0 )

	require "lua/objects/grass"		
	grass:init( 31, g.newImage( "gfx/game/grass.png" ), 0 )
	
	require "lua/objects/sign"		
	sign:init( 29, g.newImage( "gfx/game/sign.png" ), 0 )
	sign:setHitbox( 8, 0, tile.size - 16, tile.size )
	sign:addMessage( " GiLL ! Well, finally you're here and we can begin. Read the following signs to learn how to play the game. Signed 'Mr.X' ", "map0" )
	sign:addMessage( " Jump using W / UP / SPACE.  Signed 'Mr.X' ", "map0" )
	sign:addMessage( " What you can see here is a Ulta Pro 2000 Jumppad! Just went on and wonder! Signed 'Mr.X' ", "map0" )
	sign:addMessage( " Did you know that you're able to jump in mid air once? Well now you know it. Get to the platform over there. Signed 'Mr.X' " , "map0" )
	sign:addMessage( " Do you noticed the shining device on the ground? If you went over it, it can get you back from the dead. Really! Signed 'Mr.X' ", "map0" )
	sign:addMessage( " You can also use your special jumping abilitys to reach higher places. Try it. Signed 'Mr.X' ", "map0" )
	sign:addMessage( " Do you see this sharp rocks above? You better shouldn't touch them. Signed 'Mr.X' ", "map1" )
	sign:addMessage( " It get's pretty dark here. You should be careful. Signed 'Mr.X' ", "map2" )
	sign:addMessage( " Signed 'Mr.X' ", "map1" )
	
	require "lua/objects/jumppad"
	jumppad:init( 28, g.newImage( "gfx/game/jumppad.png" ), 0 )
	jumppad:addParticles( "collide", tile.size / 2, 8, 100, 100, 100, 150 )
	jumppad:setHitbox( 8, 14, tile.size / 2, 2 )
	
	require "lua/objects/spikepad"
	spikepad:init( 34, g.newImage( "gfx/game/spikepad.png" ), 100 )
	spikepad:addParticles( "collide", tile.size / 2, 8, 200, 20, 20, 150 )
	-- hitbox was set in the spikepad.lua

	require "lua/objects/spike"
	spike:init()
	
	require "lua/objects/checkpoint"
	checkpoint:init( 30, g.newImage( "gfx/game/checkpoint.png" ), 0 )
	checkpoint:setHitbox( 8, 0, tile.size - 16, 6 )
	
	require "lua/objects/conveyor"		
	conveyor:init( 36, g.newImage( "gfx/game/conveyor.png" ), 0 )
	conveyor:setHitbox( 0, 16, tile.size, 16 )
	
end

function world:getSpawn()
	
	if map.spawnX ~= nil then
		return map.spawnX, map.spawnY + 1
	else
		print( "map.spawn coords doesn't exist!" )
	end
	
end

function world:nextMap()

	for i,v in ipairs( maplist ) do
		if currentmap == v then return maplist[ i + 1 ] end
	end

end

-- Load a mapfile
function world:loadMap( mapname )	-- Load maps ( used in changeMap function )

	local f = love.filesystem
	require "libs/TSerial"
	
	if f.exists( "maps/".. mapname ..".map" ) then
		local mapfile = f.newFile( "maps/".. mapname ..".map" )
		mapfile:open( "r" )
		map = TSerial.unpack( mapfile:read() ) 
		mapfile:close()
		print( "Map: ".. mapname .. " has been loaded! " )
	elseif mapname == "menu" then
		gamestate.switch( menu )
	else
		print( "File does not exist!" )
	end
	
end

-- Change the map and set the player position
function world:changeMap( mapname )

	local g = love.graphics
	
-- Load the mapfile
	if type( mapname ) == "string" then
		world:loadMap( mapname )
	else
		world:loadMap( maplist[ mapname ] )
	end
	
	currentmap = mapname
	
-- reset player position
	player:setPosition( world:getSpawn())
	if player.x ~= nil then
		camera.x, camera.y = player.x, player.y
	else
		camera.x, camera.y = 0, 0
	end
	
-- init everything
	tile:init()
	world:loadEntitys()
	
-- set the  map canvas
	world:setCanvas()

end

-- set the canvas for the current map
function world:setCanvas()

	local m = map
	local g = love.graphics
	
	canvas = g.newCanvas( map.width, map.height )
	lightmask = g.newCanvas( map.width, map.height )
	canvas:setFilter( "nearest", "nearest" )
	lightmask:setFilter( "nearest", "nearest" )
	
	g.setCanvas( canvas )
	
	for i, v in ipairs( m ) do -- NOT REALLY GOOD! ( THE v.Type == code )
		local drawTile = 0
		if v.Type == 24 then drawTile = 20 -- replace the spikewheel1 with a blank background / temp.
		elseif v.Type == 27 then drawTile = 17 -- player spawnpoint replaced with blue background
		elseif v.Type == 28 or v.Type == 29 or v.Type == 30 or v.Type == 31 or v.Type == 34 or v.Type == 35 or v.Type == 37 or v.Type == 38 or v.Type == 39 or v.Type == 40 then drawTile = 17 -- player spawnpoint replaced with blue background
		else drawTile = v.Type
		end
		g.setColor( 255, 255, 255 )
		g.drawq( tile.img, tile.table[ drawTile ], v.x, v.y )
	end
	
	g.setCanvas()

end

function world:update( dt )

	local g = love.graphics

-- update entitys
	for i = 1, # self.entityList do
		self.entityList[i]:update( dt )
	end
	
-- Dynamic lighting
	self.lightUpdateTimer = self.lightUpdateTimer + dt
	
	if self.lightUpdateTimer >= .045 and fancylighting then
	local w, h, scale = lightmaskimg:getWidth(), lightmaskimg:getHeight(), 1.75
	self.lightFlickerTimer = self.lightFlickerTimer + dt

	if self.lightFlickerTimer > .01 then
		self.lightFlickerTimer = 0
	elseif self.lightFlickerTimer > .005 then
		scale = 1.77
	end

	--Felix: update the canvas
	lightmask:clear(0,0,0,255) --Felix: clear the canvas to a black colour
	g.setCanvas( lightmask )
	local m = map
	g.setBlendMode( "additive" )
	g.setBackgroundColor( 255, 255, 255 )
	g.setColor( 255, 255, 25 )
	for i,v in ipairs( m ) do
		if v.Type == 23 then
			g.draw( lightmaskimg, v.x + 32 / 2 - (w * ( scale - 1 )) / 2 , v.y + 32 / 2 - (h * ( scale - 1 )) / 2, 0, scale - 1, scale - 1 )
		end
	end
	
	if player.x ~= nil and not player.dead then 
	posX, posY = player.x + 32 / 2 - (w * 1.4) / 2, player.y + 32 / 2 - (h * 1.4) / 2
	end
	
	g.setColor( 255, 255, 255 )
	g.draw( lightmaskimg, posX, posY, 0, 1.4, 1.4 ) --Felix: draw the light mask image at the mouse position

	g.setCanvas()
	self.lightUpdateTimer = 0
	end
	
end

function world:draw()

	local g = love.graphics

-- draw map and lighting system if activated
	if fancylighting then g.setBlendMode( "multiplicative" ) end
	g.draw( canvas, 0, 0 )
	g.setBlendMode( "alpha" )
-- draw entitys
	g.setColor( 255, 255, 255 )
	for i = 1, # self.entityList do
		self.entityList[i]:draw()
	end
	if particles ~= nil then
		particles:draw()
	end
	if fancylighting then
		g.setBlendMode( "multiplicative" )
		g.setColor( 255, 255, 255 )
		g.draw( lightmask, 0, 0 )
		g.setBlendMode( "alpha" )
	end
	
end