-- main lua

-- Debug ?
	local debug_mode = true

function love.load()

-- Game version
	local versioninfo = "0.0.8c(pre-alpha)"
	
	local g = love.graphics
	
-- Screen width and screen height	
	width, height = g.getWidth(), g.getHeight()

-- option variables / get saved later
	volume = 50	
	scale = 1
	fullscreen = false
	vsync = false
	fancylighting = true
	
	slowmotiontimer = 0

-- set the window caption
	g.setCaption( "Gill(WIP) | Version:  " .. versioninfo )
-- set graphics mode
	--g.setMode( width, height, fullscreen, vsync, 0 )
-- set the default filter
	g.setDefaultImageFilter( "nearest", "nearest" )

-- Load / set Fonts	
	font_small = g.newFont ( "font.ttf", 31 )
	font_normal = g.newFont ( "font.ttf", 36 )
	font_large = g.newFont ( "font.ttf", 61 )
	font_xl = g.newFont ( "font.ttf", 115 )
	font_xxl = g.newFont ( "font.ttf", 179 )

-- Require the libs
	Class = require "libs/class"
	gamestate = require "libs/gamestate"
	require "libs/AnAL"
	require "libs/camera"


-- Require the game lua's
	require "lua/gs/intro"
	require "lua/fader"
	require "lua/soundmanager"
	
	fader:init()	
	gamestate.switch( intro )
	gamestate.init()
	soundmanager:init()
	camera:init()
	camera:setScale( 1, 1 )

end

function love.update( dt )

	gamestate.update( dt )
	fader:update( dt )
	soundmanager:update( dt )
	camera:shakeUpdate(dt)
	
end

function love.draw()

	local g = love.graphics
	

	g.push()
	g.scale( scale, scale )

	gamestate.draw()
	fader:draw()
	g.setFont( font_small )
	g.pop()

	if debug_mode then
		g.setColor( 255, 255, 255 )
		g.print( "FPS: ".. tostring( love.timer.getFPS() ), 10, 0 + 16 * 1 )
		g.print( "Memory usage: ".. tostring( math.floor( collectgarbage("count") ) ) .. " kb ", 10, 0 + 16 * 2 )
	end

end

function love.keypressed( key )

	gamestate.keypressed( key )
	soundmanager:keypressed( key )
	
	if key == "p" then pause = not pause end

end

function love.mousepressed( x, y, b )

	gamestate.mousepressed( x, y, b )
	menuItem:mousepressed( x, y, b )

end

function boxCollide(rect1, rect2)
	local rect1x2 , rect1y2 , rect2x2 , rect2y2 = rect1.x + rect1.w, rect1.y + rect1.h, rect2.x + rect2.w , rect2.y + rect2.h
	return rect1.x < rect2x2 and rect1x2 > rect2.x and rect1.y < rect2y2 and rect1y2 > rect2.y
end

function visible(object)
	return 	object.x > camera.x - tile.size and object.x < camera.x + width
			and object.y > camera.y - tile.size and object.y < camera.y + height 
end

function love.focus( f )
	if not f then pause = true end
end